/****************************************************************************
* Title                 :   STEPPER HAL
* Filename              :   stepper_hal.c
* Author                :   MSV
* Origin Date           :   30/01/2016
* Notes                 :   Hardware layer
*****************************************************************************/
/**************************CHANGE LIST **************************************
*
*    Date    Software Version    Initials   Description
*  30/01/16     .1                  MSV     Interface Created.
*
*****************************************************************************/

/**
 * @file stepper_hal.c
 * @brief <h3> HAL Layer </h3>
 *
 * @par
 * Stepper motor pin access.
 */

/******************************************************************************
* Includes
*******************************************************************************/

#include "__STEPPER4_HAL.h"

/******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/

/******************************************************************************
* Module Preprocessor Macros
*******************************************************************************/

/******************************************************************************
* Module Typedefs
*******************************************************************************/

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/

#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
extern sfr sbit STEPPER4_PIN_1;      // ovde sam dodao CIC prefiks jer su globalno dostupna imena, treba proveriti sta je koji
extern sfr sbit STEPPER4_PIN_2;      // pin i dodati neki opis u ime
extern sfr sbit STEPPER4_PIN_3;
extern sfr sbit STEPPER4_PIN_4;
extern sfr sbit STEPPER4_EN_PIN;        // sufiks za pinove, pretpostavljam da su pinovi PROVERITI!!!!!!!!!!!!
extern sfr sbit STEPPER4_DIR_PIN;
#endif

void STEPPER4_init_hal                 //    prvo init jer je glagol
(
        uint16_t cpuMHz      // upotreba _ je zabranjena kod argumenta,
)
{
#if defined( __MIKROC_PRO_FOR_ARM__ )
    RCC_APB1ENR.TIM2EN = 1;
    TIM2_CR1.CEN = 0;
    TIM2_PSC = ( cpuMHz );
    TIM2_DIER.UIE = 1;
    TIM2_CR1.CEN = 1;
 /* RCC_APB1ENR.TIM2EN = 1;
  TIM2_CR1.CEN = 0;
  TIM2_PSC = 0;
  TIM2_ARR = 71;
  TIM2_DIER.UIE = 1;
  TIM2_CR1.CEN = 1;  */
#elif defined( __MIKROC_PRO_FOR_PIC__ )
    T5CON = 0x21;
    TMR5H = 0;
    TMR5L = 0;
    
#endif
}

uint16_t STEPPER4_readMicros_hal   // prvo glagol, mozda moze i countMicros_hal?
(
        //void                  // void kao argument je zabranjen
)
{
    uint16_t res = 0;
    
#if defined( __MIKROC_PRO_FOR_ARM__ )
    res = TIM2_CNT;
#elif defined( __MIKROC_PRO_FOR_PIC__ )
    res = TMR5H;
    res <<= 8;
    res |= TMR5L;
#endif

    return res;
}

void STEPPER4_write_hal                // zamena mesta
(
        uint8_t pin,
        uint8_t value
)
{
    switch ( pin )
    {
        case 0:
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
    STEPPER4_PIN_1 = value;
#endif
        break;
        case 1:
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
    STEPPER4_PIN_2 = value;
#endif
        break;
        case 2:
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
    STEPPER4_PIN_3 = value;
#endif
        break;
        case 3:
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
    STEPPER4_PIN_4 = value;
#endif
        break;
        default :
        break;
    };
}

void STEPPER4_setDirection_hal       // dodao set
(
        uint8_t value
)
{
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
    STEPPER4_DIR_PIN = value;
#endif
}

void STEPPER4_enable_hal        // zamena mesta
(
        uint8_t value
)
{
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_PIC__ )   || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ )
    STEPPER4_EN_PIN = value;
#endif
}

/*************** END OF FUNCTIONS *********************************************/