/****************************************************************************
* Title                 :   CLICKNAME CLICK
* Filename              :   CLICKNAME_hw.h
* Author                :   MSV
* Origin Date           :   30/01/2016
* Notes                 :   Hardware layer
*****************************************************************************/
/**************************CHANGE LIST **************************************
*
*    Date    Software Version    Initials   Description
*  30/01/16     .1                  MSV     Interface Created.
*
*****************************************************************************/
/**
 * @file CLICKNAME_hw.c
 * @brief <h3> Hardware Layer </h3>
 */
/******************************************************************************
* Includes
*******************************************************************************/

#include "__STEPPER4_Lib.h"

/******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/

#define LOW         0
#define HIGH        1

/******************************************************************************
* Module Preprocessor Macros
*******************************************************************************/

#define MAX( a, b ) ( ( (a) >= (b) ) ? (a) : (b) )

#define CONSTRAIN( amt, low, high ) ( (amt)<(low) ? (low) : ( (amt)>(high) ? (high) : (amt) ) )

/******************************************************************************
* Module Typedefs
*******************************************************************************/

/**
 * @enum stepper_dir_t
 * @brief Direction Indicator
 *
 * + DIR_CCW - Counter Clockwise
 * + DIR_CW - Clockwise
 */
typedef enum
{
    STEPPER4_DIR_CCW                 = 0,
    STEPPER4_DIR_CW                  = 1

} T_STEPPER4_direction;

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/

static void ( *fpfwd )(  );               //sve su staticke pa nema prefiksa ni _
static void ( *fpbkwd )(  );

static uint8_t      pins;

static int32_t      n;
static float        c0;
static float        cn;
static float        cmin;
static int8_t       dir;
static uint8_t      iface;
static uint8_t      pin[ 4 ];
static int32_t      pos_cur;
static int32_t      pos_tar;
static float        speed;
static float        speed_max;
static float        acc;
static float        sqrt_twoa;
static uint32_t     step_len;
static uint32_t     st_last;
static uint16_t     pw_min;

static int8_t         en_inv;
static int8_t         en_pin;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

static void compute_speed( );

static void set_out_pins(uint8_t mask);

static void set_step(int32_t step);  // dodao set_

static void set_step0( );

static void set_step1( );

static void set_step2(int32_t step);

static void set_step3(int32_t step);

static void set_step4(int32_t step);

static void set_step6(int32_t step);

static void set_step8(int32_t step);

/******************************************************************************
* Private Function Definitions
*******************************************************************************/

static void compute_speed( )
{
    int32_t dis = STEPPER4_measure_dist( );
    int32_t sts = ( int32_t )( ( speed * speed ) / ( 2.0f * acc ) );

    if( dis == 0 && sts <= 1 )
    {
        step_len = 0;
        speed = 0;
        n = 0;

        return;
    }

    if( dis > 0 )
    {
        if( n > 0 )
        {
            if( ( sts >= dis ) || dir == STEPPER4_DIR_CCW )
                n = sts;
        }
        else if( n < 0 )
        {
            if( ( sts < dis ) && dir == STEPPER4_DIR_CW )
                n = -n;
        }
    }
    else if( dis < 0 ) {

        if( n > 0 )
        {
            if( ( sts >= -dis ) || dir == STEPPER4_DIR_CW )
                n = sts;
        }
        else if( n < 0 )
        {
            if( ( sts < -dis ) && dir == STEPPER4_DIR_CCW )
                n = -n;
        }
    }

    if( n == 0 )
    {
        cn = c0;
        dir = ( dis > 0 ) ? STEPPER4_DIR_CW : STEPPER4_DIR_CCW;
    }
    else {

        cn = cn - ( ( 2.0 * cn ) / ( ( 4.0 * n ) + 1 ) );
        cn = MAX( cn, cmin );
    }

    n++;
    step_len = cn;
    speed = 1000000.0 / cn;

    if( dir == STEPPER4_DIR_CCW )
        speed = -speed;
}

static void set_out_pins(uint8_t mask)
{
    uint8_t i;

    for ( i = 0; i < pins; i++ )
        STEPPER4_write_hal( i, ( mask & ( 1 << i ) ) ? HIGH : LOW );
}

static void step(int32_t step)
{
    switch ( iface )
    {
        case STEPPER4_mif_FUNCTION:

            set_step0();

        break;
        case STEPPER4_mif_DRIVER:

            set_step1();

        break;
        case STEPPER4_mif_FULL2WIRE:

            set_step2( step );

        break;
        case STEPPER4_mif_FULL3WIRE:

            set_step3( step );

        break;
        case STEPPER4_mif_FULL4WIRE:

            set_step4( step );

        break;
        case STEPPER4_mif_HALF3WIRE:

            set_step6( step );

        break;
        case STEPPER4_mif_HALF4WIRE:

            set_step8( step );

        break;
    }
}

static void set_step0( )
{
    if( speed > 0 )
        fpfwd();
    else
        fpbkwd();
}

static void set_step1( )
{
    set_out_pins( dir ? 0b10 : 0b00 );
    set_out_pins( dir ? 0b11 : 0b01 );

    Delay_us( 10 );

    set_out_pins( dir ? 0b10 : 0b00 );
}

static void set_step2(int32_t step)
{
    switch ( step & 0x03 )
    {
        case 0 :

            set_out_pins( 0b10 );

        break;
        case 1 :

            set_out_pins( 0b11 );

        break;
        case 2 :

            set_out_pins( 0b01 );

        break;
        case 3 :

            set_out_pins( 0b00 );

        break;
    }
}

static void set_step3(int32_t step)
{
    switch ( step % 3 )
    {
        case 0 :

            set_out_pins( 0b100 );

        break;
        case 1 :

            set_out_pins( 0b001 );

        break;
        case 2 :

            set_out_pins( 0b010 );

        break;
    }
}

static void set_step4(int32_t step)
{
    switch ( step & 0x03 )
    {
        case 0 :

            set_out_pins( 0b0101 );

        break;
        case 1 :

            set_out_pins( 0b0110 );

        break;
        case 2 :

            set_out_pins( 0b1010 );

        break;
        case 3 :

            set_out_pins( 0b1001 );

        break;
    }
}

static void set_step6(int32_t step)
{
    switch ( step % 6 )
    {
        case 0 :

            set_out_pins( 0b100 );

        break;
        case 1 :

            set_out_pins( 0b101 );

        break;
        case 2 :

            set_out_pins( 0b001 );

        break;
        case 3 :

            set_out_pins( 0b011 );

        break;
        case 4 :

            set_out_pins( 0b010 );

        break;
        case 5 :

            set_out_pins( 0b110 );

        break;
    }
}

static void set_step8(int32_t step)
{
    switch ( step & 0x07 )
    {
        case 0 :

            set_out_pins( 0b0001 );

        break;
        case 1 :

            set_out_pins( 0b0101 );

        break;
        case 2 :

            set_out_pins( 0b0100 );

        break;
        case 3 :

            set_out_pins( 0b0110 );

        break;
        case 4 :

            set_out_pins( 0b0010 );

        break;
        case 5 :

            set_out_pins( 0b1010 );

        break;
        case 6 :

            set_out_pins( 0b1000 );

        break;
        case 7:

            set_out_pins( 0b1001 );

        break;
    }
}

/******************************************************************************
* Public Function Definitions
*******************************************************************************/
void STEPPER4_init
(
        T_STEPPER4_motor_iface ifacea,
        uint16_t mcuFreq,
        int8_t use_en_pin,  //f        // kako da bude jedna imenica?
        int8_t use_inv_pin,    //f
        int8_t enable_motor       //t
)
{
    //int i;       // ovo ne vidim gde se koristi

    STEPPER4_init_hal( mcuFreq );

    iface = ifacea;
    pos_cur = 0;
    pos_tar = 0;
    speed = 0.0;
    speed_max = 1.0;
    acc = 0.0;
    sqrt_twoa = 1.0;
    step_len = 0;
    pw_min = 1;
    st_last = 0;
    en_pin = use_en_pin;
    n = 0;
    c0 = 0.0;
    cn = 0.0;
    cmin = 1.0;
    dir = STEPPER4_DIR_CCW;

    if ( iface == STEPPER4_mif_FULL4WIRE || iface == STEPPER4_mif_HALF4WIRE )
        pins = 4;
    else if ( iface == STEPPER4_mif_FULL3WIRE || iface == STEPPER4_mif_HALF3WIRE )
        pins = 3;
    else
        pins = 2;

    if( enable_motor )     ;
       // STEPPER4_PIN_3 = 1;//stepper_enable_outputs();  //??????????????????  ova fja stvarno treba!!!!!!

    STEPPER4_set_acc( 1 );    // ovo ne vidim da treba
}

void STEPPER4_init_fp(void(*fpforward )( ), void(*fpbackward)( ))
{
    fpfwd = fpforward;
    fpbkwd = fpbackward;
}

void STEPPER4_move_to(int32_t position_abs)
{
    if( pos_tar != position_abs )
    {
        pos_tar = position_abs;
        compute_speed();
    }
}

void STEPPER4_move(int32_t position_rel)
{
    STEPPER4_move_to(pos_cur + position_rel);
}

int8_t STEPPER4_run( )
{
    if( STEPPER4_run_speed() )
        compute_speed();

    return ( speed != 0.0 ) || ( STEPPER4_measure_dist( ) != 0 );
}

int8_t STEPPER4_run_speed( )
{
    // DBG
    //char txt[ 20 ];
    
    uint32_t next_step_time;
    uint32_t time;

    if( !step_len )
        return false;

    time = STEPPER4_readMicros_hal();
    next_step_time = st_last + step_len;

    if( ( ( next_step_time >= st_last ) && ( ( time >= next_step_time ) || ( time < st_last ) ) ) || \
      ( ( next_step_time < st_last ) && ( ( time >= next_step_time ) && ( time < st_last ) ) ) )
    {
        if ( dir == STEPPER4_DIR_CW )
            pos_cur += 1;
        else
            pos_cur -= 1;


        step( pos_cur );

        st_last = time;

        return true;
    }
    else {

        return false;
    }
}

int32_t STEPPER4_measure_dist( )
{
    return pos_tar - pos_cur;
}

int32_t STEPPER4_read_tar_pos( )
{
    return pos_tar;
}

int32_t STEPPER4_read_cur_pos()
{
    return pos_cur;
}

void STEPPER4_set_cur_pos(int32_t position)
{
    pos_tar = pos_cur = position;
    n = 0;
    step_len = 0;
    speed = 0;
}

void STEPPER4_set_max_speed(float speed)
{
    if( speed_max != speed )
    {
        speed_max = speed;
        cmin = 1000000.0 / speed;

        if( n > 0 )
        {
            n = ( int32_t )( ( speed * speed ) / ( 2.0 * acc ) );
            compute_speed();
        }
    }
}

void STEPPER4_set_acc(float acceleration)
{
    if( acc == 0.0 )
        return;

    if( acc != acceleration )
    {
        n = n * ( acc / acceleration );
        c0 = 0.676 * sqrt( 2.0 / acceleration ) * 1000000.0;
        acc = acceleration;

        compute_speed();
    }
}

void STEPPER4_set_speed(float speedf) // dodao sam f da bi se razlikovala od globalne speed
{
    if( speedf == speed )
        return;

    speedf = CONSTRAIN( speedf, -speed_max, speed_max );

    if( speedf == 0.0 )
    {
        step_len = 0;
    }
    else {

        step_len = fabs( 1000000.0 / speedf );
        dir = ( speedf > 0.0 ) ? STEPPER4_DIR_CW : STEPPER4_DIR_CCW;
    }

    speed = speedf;
}

float STEPPER4_read_speed( )
{
    return speed;
}

void STEPPER4_disable_outputs( )
{
    if( !iface )
        return;

    if( en_pin )
        STEPPER4_enable_hal( 0 );
    else
        return;
}

void STEPPER4_run_to_position( )
{
    while( STEPPER4_run() );
}

int8_t STEPPER4_run_speed_to_position( )
{
    if( pos_tar == pos_cur )
        return false;

    if( pos_tar > pos_cur )
        dir = STEPPER4_DIR_CW;
    else
        dir = STEPPER4_DIR_CCW;

    return STEPPER4_run_speed();
}

void STEPPER4_run_to_new_position(int32_t position)
{
    STEPPER4_move_to( position );
    STEPPER4_run_to_position();
}

void STEPPER4_stop( )
{
    int32_t sts;

    if( speed != 0.0 )
    {
        sts = ( int32_t )( ( speed * speed ) / ( 2.0 * acc ) ) + 1;

        if( speed > 0 )
            STEPPER4_move( sts );
        else
            STEPPER4_move( -sts );
    }

    if( en_pin )
        STEPPER4_enable_hal( 1 );
}

void STEPPER4_set_min_pulse_width(uint16_t width)
{
    pw_min = width;
}

int8_t STEPPER4_is_running( )
{
    return !( speed == 0.0 && pos_tar == pos_cur );

}

/*************** END OF FUNCTIONS *********************************************/